@@ -3,7 +3,7 @@
  *
  * See Copyright for the status of this software.
  *
- * $Id: rpmopen.c,v 1.103 2001/08/14 10:40:56 veillard Exp $
+ * $Id: rpmopen.c,v 1.108 2003/12/16 16:54:55 bellet Exp $
  */
 
 #include <config.h>
@@ -25,10 +25,21 @@
 #include <rpm/rpmlib.h>
 #include <rpm/rpmmacro.h>
 #include <rpm/rpmdb.h>
-//#include <rpm/rpmts.h>
+#include <rpm/rpmts.h>
+#ifdef HAVE_RPM42
+#include <rpm/rpmfi.h>
+#endif
 
 int readLead(FD_t fd, /*@out@*/struct rpmlead *lead);
+#ifdef HAVE_RPM42
+typedef enum sigType_e {
+    RPMSIGTYPE_HEADERSIG= 5     /*!< Header style signature */
+} sigType;
+
+int rpmReadSignature(FD_t fd, Header *sighp, sigType sig_type, const char ** msg);
+#else
 int rpmReadSignature(FD_t fd, /*@out@*/ Header *header, short sig_type);
+#endif
 
 #if defined(WITH_GPG)
 #include <sys/wait.h>
@@ -850,12 +861,23 @@ rpmAnalyze(char *path, char *nameRpm, const char *sigtarget,
     }
     ENTRY_CLEANUP(p);
 
-#if defined(RPMTAG_FILENAMES)
+#ifdef HAVE_RPM42
+    const char *fname;
+    rpmfi fi;
+    fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, 0);
+    fi = rpmfiInit(fi, 0);
+    rpm->extra->filelist = NULL;	/* No filelist in the package ! */
+    char *ptr = buffer;
+
+        while (rpmfiNext(fi) >= 0) {
+            fname = rpmfiFN(fi);
+#else
+  #if defined(RPMTAG_FILENAMES)
     val = getTagNumber("RPMTAG_FILENAMES");
     headerGetEntry(h, val, &type, &p, &count);
-#else
+  #else
     rpmBuildFileList(h, (const char ***) &p, &count);
-#endif
+  #endif
 
     if (count == 0) {
         rpm->extra->filelist = NULL;    /* No filelist in the package ! */
@@ -863,6 +885,7 @@ rpmAnalyze(char *path, char *nameRpm, const char *sigtarget,
         char *ptr = buffer;
 
         for (i = 0; i < count; i++) {
+#endif
             if ((ptr - buffer) > (buffer_size - 1024)) {
                 int delta = ptr - buffer;
 
@@ -875,11 +898,20 @@ rpmAnalyze(char *path, char *nameRpm, const char *sigtarget,
                 }
                 ptr = &buffer[delta];
             }
+#ifdef HAVE_RPM42
+            ptr += snprintf(ptr, buffer_size, "%s\n", fname);
+#else
             ptr += snprintf(ptr, buffer_size, "%s\n", ((char **) p)[i]);
+#endif
         }
+#ifdef HAVE_RPM42
+        fi = rpmfiFree(fi);
+#endif
         rpm->extra->filelist = xmlStrdup(buffer);
+#ifndef HAVE_RPM42
     }
     ENTRY_CLEANUP(p);
+#endif
 
 #ifdef WITH_GPG
     rpmResolveSignatures(path, nameRpm, sigtarget, rpm);
@@ -1016,6 +1048,9 @@ rpmOpen(char *nameRpm, rpmDirPtr dir, rpmSubdirPtr tree)
     /* read the RPM header */
 #if defined(_RPMVSF_NODIGESTS)
     {	rpmts ts = rpmtsCreate();
+	/* do no verifications except the SHA1 header */
+	/* for performance reasons (see redhat bugzilla #83320) */
+	rpmtsSetVSFlags(ts, ~RPMVSF_NOSHA1HEADER);
 	rc = rpmReadPackageFile(ts, fd, buffer, &h);
 	ts = rpmtsFree(ts);
     }
@@ -1041,6 +1076,16 @@ rpmOpen(char *nameRpm, rpmDirPtr dir, rpmSubdirPtr tree)
             fprintf(stderr, "query of %s failed\n", nameRpm);
             fdClose(fd);
             return (NULL);
+        case 4:
+	    fprintf(stderr, "%s: Error %d Found and Ignored\n", nameRpm, rc);
+	    if (!h) {
+		fprintf(stderr,
+			"packages cannot be queried\n");
+		fdClose(fd);
+		return (NULL);
+	    }
+	    isSource = (headerIsEntry(h, RPMTAG_SOURCEPACKAGE) ? 1 : 0);
+	    break;
         default:
             fprintf(stderr, "rpmReadPackageHeader(%s): unknown error %d\n",
                     nameRpm, rc);
@@ -1072,7 +1117,11 @@ rpmOpen(char *nameRpm, rpmDirPtr dir, rpmSubdirPtr tree)
                     nameRpm);
             break;
         default:
+#ifdef HAVE_RPM42
+            if (rpmReadSignature(fd, &s, lead.signature_type, NULL)) {
+#else
             if (rpmReadSignature(fd, &s, lead.signature_type)) {
+#endif
                 fprintf(stderr, "rpmReadSignature failed\n");
                 fdClose(fd);
                 return (NULL);
